# Copyright 2020-2023 The TFP CausalImpact Authors
# Copyright 2014 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utils for printing impact summaries."""

from typing import Optional
from jinja2 import Template

summary_text = """
{% macro CI(alpha) %}{{(((1 - alpha) * 100) | string).rstrip('0').rstrip('.')}}% CI{% endmacro -%}
{% macro add_remaining_spaces(n) %}{{' ' * (19 -n)}}{% endmacro -%}
Posterior Inference {CausalImpact}
                          Average            Cumulative
Actual                    {{summary.average.actual | round(1)}}{{add_remaining_spaces(summary.average.actual | round(1) | string | length)}}{{summary.cumulative.actual | round(1)}}
Prediction (s.d.)         {{summary.average.predicted | round(1)}} ({{summary.average.predicted_sd | round(2)}}){{add_remaining_spaces(summary.average.predicted | round(1) | string | length + 3 + summary.average.predicted_sd | round(2) | string | length)}}{{summary.cumulative.predicted | round(1)}} ({{summary.cumulative.predicted_sd | round(2)}})
{{CI(alpha)}}                    [{{summary.average.predicted_lower | round(1)}}, {{summary.average.predicted_upper | round(1)}}]{{add_remaining_spaces(4 + summary.average.predicted_lower | round(1) | string | length + summary.average.predicted_upper | round(1) | string | length)}}[{{summary.cumulative.predicted_lower | round(1)}}, {{summary.cumulative.predicted_upper | round(1)}}]

Absolute effect (s.d.)    {{summary.average.abs_effect | round(1)}} ({{summary.average.abs_effect_sd | round(2)}}){{add_remaining_spaces(3 + summary.average.abs_effect | round(1) | string | length + summary.average.abs_effect_sd | round(2) | string | length)}}{{summary.cumulative.abs_effect | round(1)}} ({{summary.cumulative.abs_effect_sd | round(2)}})
{{CI(alpha)}}                    {{[summary.average.abs_effect_lower | round(1), summary.average.abs_effect_upper | round(1)] | sort}}{{add_remaining_spaces(4 + summary.average.abs_effect_lower | round(1) | string | length + summary.average.abs_effect_upper | round(1) | string | length)}}{{[summary.cumulative.abs_effect_lower | round(1), summary.cumulative.abs_effect_upper | round(1)] | sort}}

Relative effect (s.d.)    {{'{0:.1%}'.format(summary.average.rel_effect)}} ({{'{0:.1%}'.format(summary.average.rel_effect_sd|float)}}){{add_remaining_spaces(3 + '{0:.1%}'.format(summary.average.rel_effect) | length + '{0:.1%}'.format(summary.average.rel_effect_sd|float)| string | length)}}{{'{0:.1%}'.format(summary.cumulative.rel_effect)}} ({{'{0:.1%}'.format(summary.cumulative.rel_effect_sd | round(2)|float)}})
{{CI(alpha)}}                    [{{'{0:.1%}'.format([summary.average.rel_effect_lower, summary.average.rel_effect_upper]|min)}}, {{'{0:.1%}'.format([summary.average.rel_effect_upper, summary.average.rel_effect_lower]|max)}}]{{add_remaining_spaces(4 + '{0:.1%}'.format([summary.average.rel_effect_lower, summary.average.rel_effect_upper]|min)|length + '{0:.1%}'.format([summary.average.rel_effect_upper, summary.average.rel_effect_lower]|max)|length)}}[{{'{0:.1%}'.format([summary.cumulative.rel_effect_lower, summary.cumulative.rel_effect_upper]|min)}}, {{'{0:.1%}'.format([summary.cumulative.rel_effect_upper, summary.cumulative.rel_effect_lower]|max)}}]

Posterior tail-area probability p: {{p_value|round(3)}}
Posterior probability of an effect: {{'{0:.2%}'.format(1 - p_value)}}

For more details run the command: summary(impact, output_format="report")
"""

report_text = """
{% set detected_sig = not (summary.average.rel_effect_lower < 0 and summary.average.rel_effect_upper > 0) -%}
{% set positive_sig = summary.average.rel_effect > 0 -%}
{% macro CI(alpha) %}{{(((1 - alpha) * 100) | string).rstrip('0').rstrip('.')}}%{% endmacro -%}
Analysis report {CausalImpact}


During the post-intervention period, the response variable had
an average value of approx. {{summary.average.actual | round(1)}}. {% if detected_sig -%}By contrast, in{% else %}In{% endif %} the absence of an
intervention, we would have expected an average response of {{summary.average.predicted | round(1)}}.
The {{CI(alpha)}} interval of this counterfactual prediction is [{{summary.average.predicted_lower | round(1)}}, {{summary.average.predicted_upper | round(1)}}].
Subtracting this prediction from the observed response yields
an estimate of the causal effect the intervention had on the
response variable. This effect is {{summary.average.abs_effect | round(1)}} with a {{CI(alpha)}} interval of
{{[summary.average.abs_effect_lower | round(1), summary.average.abs_effect_upper | round(1)] | sort}}. For a discussion of the significance of this effect,
see below.


Summing up the individual data points during the post-intervention
period (which can only sometimes be meaningfully interpreted), the
response variable had an overall value of {{summary.cumulative.actual | round(1)}}.
{% if detected_sig %}By contrast, had{% else %}Had{% endif %} the intervention not taken place, we would have expected
a sum of {{summary.cumulative.predicted| round(1)}}. The {{CI(alpha)}} interval of this prediction is {{[summary.cumulative.predicted_lower | round(1), summary.cumulative.predicted_upper | round(1)]|sort}}.


The above results are given in terms of absolute numbers. In relative
terms, the response variable showed {% if positive_sig %}an increase of +{% else %}a decrease of {% endif %}{{'{0:.1%}'.format(summary.average.rel_effect)}}. The {{CI(alpha)}}
interval of this percentage is [{{'{0:.1%}'.format([summary.average.rel_effect_lower, summary.average.rel_effect_upper] | min)}}, {{'{0:.1%}'.format([summary.average.rel_effect_upper, summary.average.rel_effect_lower]|max)}}].
{% if detected_sig and positive_sig %}

This means that the positive effect observed during the intervention
period is statistically significant and unlikely to be due to random
fluctuations. It should be noted, however, that the question of whether
this increase also bears substantive significance can only be answered
by comparing the absolute effect ({{summary.average.abs_effect | round(1)}}) to the original goal
of the underlying intervention.
{% elif detected_sig and not positive_sig %}

This means that the negative effect observed during the intervention
period is statistically significant.
If the experimenter had expected a positive effect, it is recommended
to double-check whether anomalies in the control variables may have
caused an overly optimistic expectation of what should have happened
in the response variable in the absence of the intervention.
{% elif not detected_sig and positive_sig %}

This means that, although the intervention appears to have caused a
positive effect, this effect is not statistically significant when
considering the entire post-intervention period as a whole. Individual
days or shorter stretches within the intervention period may of course
still have had a significant effect, as indicated whenever the lower
limit of the impact time series (lower plot) was above zero.
{% elif not detected_sig and not positive_sig -%}

This means that, although it may look as though the intervention has
exerted a negative effect on the response variable when considering
the intervention period as a whole, this effect is not statistically
significant and so cannot be meaningfully interpreted.
{% endif %}
{%- if not detected_sig %}

The apparent effect could be the result of random fluctuations that
are unrelated to the intervention. This is often the case when the
intervention period is very long and includes much of the time when
the effect has already worn off. It can also be the case when the
intervention period is too short to distinguish the signal from the
noise. Finally, failing to find a significant effect can happen when
there are not enough control variables or when these variables do not
correlate well with the response variable during the learning period.
{% endif %}
{%- if p_value < alpha %}

The probability of obtaining this effect by chance is very small
(Bayesian one-sided tail-area probability p = {{p_value | round(3)}}).
This means the effect is statistically significant. It can be
considered causal if the model assumptions are satisfied.
{%- else %}

The probability of obtaining this effect by chance is p = {{'{0:.0%}'.format(p_value)}}.
This means the effect may be spurious and would generally not be
considered statistically significant.
{%- endif %}


For more details, including the model assumptions behind the method, see
https://google.github.io/CausalImpact/."""

SUMMARY_TMPL = Template(summary_text)
REPORT_TMPL = Template(report_text)


def summary(
    ci_model, output_format: str = "summary", alpha: Optional[float] = None
):
  """Get summary of impact results.

  Args:
    ci_model: CausalImpact instance, after calling `.train`.
    output_format: string directing whether to print a shorter summary
      ('summary') or a long-form description ('report').
    alpha: float for alpha level to use; must be in (0, 1).

  Raises:
    DeprecationWarning: In case `alpha` is explicitly set.

  Returns:
    Text output of summary results.
  """
  inferred_alpha = ci_model.summary.alpha.mean()
  if alpha is not None and alpha != inferred_alpha:
    raise DeprecationWarning("Supplying an argument to `alpha` is deprecated, "
                             "since it is inferred from `ci_model`. Set "
                             f"`alpha=None` to use alpha={inferred_alpha:.2f}, "
                             f"or retrain the model with alpha={alpha}.")
  alpha = inferred_alpha

  if output_format not in ["summary", "report"]:
    raise ValueError("`format` must be either 'summary' or 'report'. "
                     "Got %s" % output_format)

  if alpha <= 0. or alpha >= 1.:
    raise ValueError("`alpha` must be in (0, 1). Got %s" % alpha)

  p_value = ci_model.summary["p_value"][0]

  if output_format == "summary":
    output = SUMMARY_TMPL.render(
        summary=ci_model.summary.transpose().to_dict(),
        alpha=alpha,
        p_value=p_value)
  else:
    output = REPORT_TMPL.render(
        summary=ci_model.summary.transpose().to_dict(),
        alpha=alpha,
        p_value=p_value)

  return output



# Copyright 2020-2023 The TFP CausalImpact Authors
# Copyright 2014 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Functions for working with indices."""

import datetime
from typing import Tuple, Union

import numpy as np
import pandas as pd

InputDateType = Union[str, int, datetime.datetime]
OutputDateType = Union[int, datetime.datetime]
InputPeriodType = Tuple[InputDateType, InputDateType]
OutputPeriodType = Tuple[OutputDateType, OutputDateType]


def parse_and_validate_date_data(
    data: pd.DataFrame,
    pre_period: InputPeriodType,
    post_period: InputPeriodType,
) -> Tuple[OutputPeriodType, OutputPeriodType]:
  """Parses and validates date arguments.

  Args:
    data: Pandas `DataFrame` containing an outcome time series and optional
      feature time series.
    pre_period: Tuple of strings giving pre-period start/end date-times (the
      date-time format should match that of data.index).
    post_period: Tuple of strings giving post-period start/end date-times (the
      date-time format should match that of data.index).

  Returns:
    A converted pre_period and post_period, which are all of the same type as
    `data.index`.
  """
  pre_period = tuple([_convert_date_to_index_type(p, data) for p in pre_period])
  post_period = tuple(
      [_convert_date_to_index_type(p, data) for p in post_period])
  pre_period, post_period = _parse_and_validate_periods(pre_period, post_period,
                                                        data)
  return pre_period, post_period


def _parse_and_validate_periods(
    pre_period: OutputPeriodType, post_period: OutputPeriodType,
    data: pd.DataFrame) -> Tuple[OutputPeriodType, OutputPeriodType]:
  """Makes sure pre- and post-periods are specified correctly.

  Args:
    pre_period: strings giving pre-period start/end date-times (the date-time
      format should match that of data.index).
    post_period: strings giving post-period start/ebd date-times (the date-time
      format should match that of data.index).
    data: dataframe with a pd.DatetimeIndex that the pre/post periods are based
      on.

  Returns:
    result: tuple of checked pre/post periods

  Raises:
    ValueError: if pre_period overlaps with post_period
                if pre-period is less than 3 time points
                if the pre/post_period dates are not given in order.
  """
  # Make sure pre/post periods are specified correctly.
  checked_pre_period = _check_period(pre_period, data)
  checked_post_period = _check_period(post_period, data)

  pre_period_dates = data.index[(data.index >= checked_pre_period[0])
                                & (data.index <= checked_pre_period[1])]

  # Make sure pre/post periods make sense.
  if checked_pre_period[1] >= checked_post_period[0]:
    raise ValueError("pre_period and post_period cannot overlap.")
  if len(pre_period_dates) < 3:
    raise ValueError("pre_period must span at least 3 time points. Got %s" %
                     len(pre_period_dates))
  if checked_pre_period[1] < checked_pre_period[0]:
    raise ValueError("pre_period last number must be bigger than its first.")
  if checked_post_period[1] < checked_post_period[0]:
    raise ValueError("post_period last number must be bigger than its first.")

  return (checked_pre_period, checked_post_period)


def _check_period(period: OutputPeriodType,
                  data: pd.DataFrame) -> OutputPeriodType:
  """Checks that periods are given appropriately.

  Args:
    period: strings giving period start/end date-times (the date-time format
      should match that of data.index).
    data: input data.

  Returns:
    period: checked period list with periods converted to datetime.

  Raises:
    ValueError: if period is not of type tuple.
                if period doesn't have two elements.
                if period has None elements.
                if period date values are not present in data.
                if period not given in order.
  """
  # Check that the dates are in order.
  if period[0] > period[1]:
    raise ValueError(f"Period end must be after period start. Got {period}")

  # Allow indices (or more likely) dates that are not aligned with the original
  # data. The period is rounded to prefer shorter periods rather than partially
  # covered periods.
  period_start_idx = data.index.get_indexer([period[0]], method="bfill")[0]
  if period_start_idx == -1:
    raise ValueError("Aligned period start not found in the index.")
  period_start = data.index[period_start_idx]

  period_end_idx = data.index.get_indexer([period[1]], method="ffill")[0]
  if period_end_idx == -1:
    raise ValueError("Aligned period end not found in the index.")
  period_end = data.index[period_end_idx]

  return (period_start, period_end)


def _convert_date_to_index_type(input_date: InputDateType,
                                data: pd.DataFrame) -> OutputDateType:
  if isinstance(input_date, str):
    return pd.to_datetime(input_date)
  elif isinstance(input_date, (int, np.integer)):
    return data.index[input_date]
  elif isinstance(input_date, datetime.datetime):
    return input_date
  else:
    raise ValueError(
        f"Expected argument to be str, int, or datetime. Got {type(input_date)}"
    )



# Copyright 2019-2023 The TFP CausalImpact Authors
# Copyright 2014 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Class for storing and preparing data for modeling."""
from typing import Optional, Text, Tuple, Union

from causalimpact import indices
from causalimpact import standardize
import pandas as pd
import tensorflow as tf
import tensorflow_probability as tfp


class CausalImpactData:
    """Class for storing and preparing data for modeling.

    This class handles all of the data-related functions of CausalImpact. It
    makes sure the input data is given in an appropriate format (as a pandas
    DataFrame or convertible to pd.DataFrame), checks that the given
    outcome/feature column names exist, and splits the data into the pre-period
    and post-period based on the given treatment start time OR tuples defining
    the start/end points of the pre/post periods.

    If the pre- and post-periods do not cover the entire timespan of
    the data, the excluded portions will be used for plotting but NOT for
    fitting the model or calculating impact statistics.

    Note that the default is to standardize the pre-period data to have mean zero
    and standard deviation one. The standardization is done for each column
    separately, and then applied to the post-period data using each column's pre-
    period mean and standard deviation. The mean and standard deviation for the
    outcome column are stored so that the inferences can be back-transformed to
    the original scale.


    Attributes:
      data: Pandas DataFrame of timeseries data.
      pre_period: Start and end value in data.index for pre-intervention.
      post_period: Start and end value in data.index for post-intervention.
      outcome_column: Timeseries being modeled. Defaults to first column of
        `data`.
      feature_columns: Subset of data.columns used as covariates. `None` in case
        there are no covariates. Defaults to all non-outcome columns (or `None` if
        there are none).
      standardize_data: Boolean: Whether covariates and outcome were scaled to
        have 0 mean and 1 standard deviation.
      pre_data: Subset of `data` from `pre_period`. This is unscaled.
      after_pre_data: Subset of `data` from after the `pre_period`. The time
        between pre-period and post-period should still be forecasted to make
        accurate post-period predictions. Additionally, users are interested in
        after post-period predictions. This is unscaled.
      num_steps_forecast: Number of elements (including NaN) to forecast for,
        including the post-period and time between pre-period and post-period.
      model_pre_data: Scaled subset of `data` from `pre_period` used for fitting
        the model.
      model_after_pre_data: Scaled subset of `data` from `post_period` used for
        fitting the model.
      outcome_scaler: A `standardize.Scaler` object used to transform outcome
        data.
      feature_ts: A pd.DataFrame of the scaled data over just the feature columns.
      outcome_ts: A tfp.sts.MaskedTimeSeries instance of the outcome data from the
        `pre_period`.
    """

    def __init__(self,
                 data: Union[pd.DataFrame, pd.Series],
                 pre_period: Tuple[indices.InputDateType, indices.InputDateType],
                 post_period: Tuple[indices.InputDateType, indices.InputDateType],
                 drop_post_period_nan=True,
                 outcome_column: Optional[Text] = None,
                 standardize_data=True,
                 dtype=tf.float32):
        """Constructs a `CausalImpactData` instance.

        Args:
          data: Pandas `DataFrame` containing an outcome time series and optional
            feature time series.
          pre_period: Pre-period start and end (see InputDateType).
          post_period: Post-period start and end (see InputDateType).
          outcome_column: String giving the name of the outcome column in `data`. If
            not specified, the first column in `data` is used.
          standardize_data: If covariates and output should be standardized.
          dtype: The dtype to use throughout computation.
        """
        # This is a no-op in case data is a pd.DataFrame. It is common enough to
        # pass a pd.Series that this is useful here.
        data = pd.DataFrame(data)
        self.pre_period, self.post_period = indices.parse_and_validate_date_data(
            data=data, pre_period=pre_period, post_period=post_period)
        self.data, self.outcome_column, self.feature_columns = (
            _validate_data_and_columns(data, outcome_column))
        del data  # To insure the unfiltered DataFrame is not used again.
        self.standardize_data = standardize_data
        self.pre_data = self.data.loc[(self.data.index >= self.pre_period[0])
                                      & (self.data.index <= self.pre_period[1])]
        # after_pre_data intentionally includes everything after the end of the
        # pre-period since the time between pre- and post-period needs to be
        # accounted for and we actually want to see predictions after the post
        # period.
        self.after_pre_data = self.data.loc[self.data.index > self.pre_period[1]]
        self.num_steps_forecast = len(self.after_pre_data.index)

        if self.standardize_data:
            scaler = standardize.Scaler().fit(self.pre_data)
            self.outcome_scaler = standardize.Scaler().fit(
                self.pre_data[self.outcome_column])
            self.model_pre_data = scaler.transform(self.pre_data)
            self.model_after_pre_data = scaler.transform(self.after_pre_data)
        else:
            self.outcome_scaler = None
            self.model_pre_data = self.pre_data
            self.model_after_pre_data = self.after_pre_data
        if drop_post_period_nan:
            self.model_after_pre_data = self.model_after_pre_data.dropna()

        out_ts = tf.convert_to_tensor(
            self.model_pre_data[self.outcome_column], dtype=dtype)
        self.outcome_ts = tfp.sts.MaskedTimeSeries(
            time_series=out_ts, is_missing=tf.math.is_nan(out_ts))
        if self.feature_columns is not None:
            # Here we have to use the FULL time series so that the post-period
            # feature data can be used for forecasting.
            features_pre = self.model_pre_data[self.feature_columns]
            features_post = self.model_after_pre_data[self.feature_columns]
            self.feature_ts = pd.concat([features_pre, features_post], axis=0)
            self.feature_ts["intercept_"] = 1.
        else:
            self.feature_ts = None


def _validate_data_and_columns(data: pd.DataFrame,
                               outcome_column: Optional[str]):
    """Validates data and sets defaults for feature and outcome columns.

    By default, the first column of the dataframe will be used as the outcome,
    and the rest will be used as features, but these can instead be provided.

    Args:
      data: Input dataframe for analysis.
      outcome_column: Optional string to use for the outcome.

    Raises:
      KeyError: if `outcome_column` is not in the data.
      ValueError: if `outcome_column` is constant.

    Returns:
      The validated (possibly default) data, outcome column, and feature columns.
    """

    # Check outcome column -- if not specified, default is the first column.
    if outcome_column is None:
        outcome_column = data.columns[0]
    if outcome_column not in data.columns:
        raise KeyError(f"Specified `outcome_column` ({outcome_column}) not found "
                       f"in data")

    # Make sure outcome column is not constant
    if data[outcome_column].std(skipna=True, ddof=0) == 0:
        raise ValueError("Input response cannot be constant.")

    # Feature columns are all those other than the output column. Use
    # `original_column_order` to keep track of the
    # original column order, since set(data.columns) reorders the
    # columns, which leads to problems later when subsetting and transforming.
    if data.shape[1] <= 1:
        feature_columns = None
    else:
        original_column_order = data.columns
        column_differences = set(data.columns).difference([outcome_column])
        feature_columns = [
            col for col in original_column_order if col in column_differences
        ]
    data = data[[outcome_column] + (feature_columns or [])]
    if data[outcome_column].count() < 3:  # Series.count() is for non-NaN values.
        raise ValueError("Input data must have at least 3 observations.")
    if data[feature_columns or []].isna().values.any():
        raise ValueError("Input data cannot have any missing values.")
    if not data.dtypes.map(pd.api.types.is_numeric_dtype).all():
        raise ValueError("Input data must contain only numeric values.")

    return data, outcome_column, feature_columns



# Copyright 2020-2023 The TFP CausalImpact Authors
# Copyright 2014 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for summary.py."""

import os

from absl.testing import absltest
from absl.testing import parameterized

import causalimpact as ci
import numpy as np
import pandas as pd


CURR_PATH = os.path.dirname(__file__)
TEST_PATH = os.path.join(CURR_PATH, "testdata")


def summary_data():
  """Create fake summary data."""
  data = [
      [5.343, 10.343],
      [4.343, 9.343],
      [3.343, 8.343],
      [6.343, 9.343],
      [0.001, 0.100],
      [3.343, 10.343],
      [2.343, 4.343],
      [6.343, 9.343],
      [0.001, 0.100],
      [0.123, 0.233],
      [0.143, 0.133],
      [0.343, 0.333],
      [0.001, 0.100],
      [0.001, 0.001],
      [0.100, 0.100]
  ]

  # Transpose data to get in right shape.
  summary = pd.DataFrame(
      np.array(data).T.tolist(),
      columns=[
          "actual", "predicted", "predicted_lower", "predicted_upper",
          "predicted_sd", "abs_effect", "abs_effect_lower", "abs_effect_upper",
          "abs_effect_sd", "rel_effect", "rel_effect_lower", "rel_effect_upper",
          "rel_effect_sd", "p_value", "alpha"
      ],
      index=["average", "cumulative"])
  return ci.CausalImpactAnalysis(
      series=pd.DataFrame(), summary=summary, posterior_samples=[])


class SummaryTest(parameterized.TestCase):

  @parameterized.named_parameters([
      {
          "testcase_name": "summary1",
          "p_value": 0.5,
          "rel_col_values": [0.41, -0.30, 0.30],
          "file_num": 1
      },
      {
          "testcase_name": "summary2",
          "p_value": 0.05,
          "rel_col_values": [0.41, 0.434, 0.234],
          "file_num": 2
      },
      {
          "testcase_name": "summary3",
          "p_value": 0.5,
          "rel_col_values": [-0.343, -0.434, 0.234],
          "file_num": 3
      },
      {
          "testcase_name": "summary4",
          "p_value": 0.05,
          "rel_col_values": [-0.343, -0.434, -0.234],
          "file_num": 4
      },
  ])
  def testReport(self, p_value, rel_col_values, file_num):
    ci_data = summary_data()
    ci_data.summary["p_value"] = p_value
    rel_cols = ["rel_effect", "rel_effect_lower", "rel_effect_upper"]
    ci_data.summary.loc["average", rel_cols] = rel_col_values
    output = ci.summary(ci_data, output_format="report", alpha=0.1).strip()
    with open(
        os.path.join(TEST_PATH, "test_report_text_" + str(file_num) + ".txt"),
        "r") as f:
      expected = f.read().strip()

    self.assertEqual(output, expected)

  def testSummary(self):
    ci_data = summary_data()
    ci_data.summary["p_value"] = 0.459329
    output = ci.summary(ci_data, output_format="summary", alpha=0.1).strip()
    with open(os.path.join(TEST_PATH, "test_summary_output.txt"), "r") as f:
      expected = f.read().strip()

    self.assertEqual(output, expected)


if __name__ == "__main__":
  absltest.main()
